indexing
	description: "A very simple parser to parse ids, dates and names"
	author: "Bernd Schoeller <bernd.schoeller@inf.ethz.ch>"
	date: "$Date: 2008-03-20 13:36:41 +0100 (Do, 20 Mär 2008) $"
	revision: "$Revision: 5098 $"

class
	SIMPLE_PARSER

create
	make

feature {NONE} -- Initialization

	make is
			-- Make the parser with empty input
		do
			set_input ("")
		ensure
			end_of_input: end_of_input
		end

feature -- Access

	last_id: INTEGER
		-- Last id parsed

	last_date: STRING
		-- Last date parsed

	last_name: STRING
		-- Last name parsed

	error_messages: LIST [STRING]
		-- Error messages during parsing

	input: STRING
		-- Input to parse

	position: INTEGER
		-- Current position of parsing

	item: CHARACTER is
			-- Current character in input
		require
			not_eof: not end_of_input
		do
			Result := input.item (position)
		end

feature -- Status report

	has_errors: BOOLEAN is
			-- Have there been errors during parsing?
		do
			Result := not error_messages.is_empty
		end

	end_of_input: BOOLEAN is
			-- Has the end of input been reached?
		do
			Result := position > input.count
		end

feature -- Status setting

	set_input (a_string: STRING) is
			-- Set the input to `a_string'.
		require
			not_void: a_string /= Void
		do
			input := a_string
			position := 1
			create {LINKED_LIST[STRING]} error_messages.make
		end

feature -- Cursor movement

	forth is
			-- Move the position forward
		require
			not_eof: not end_of_input
		do
			position := position + 1
		end

feature -- Parsing

	parse_id is
			-- Parse an id from the input.
		do
			last_id := 0
			from
				skip_white_space
			until
				end_of_input or not item.is_digit
			loop
				last_id := last_id * 10
				last_id := last_id + item.code - ('0').code
				forth
			end
			if not (end_of_input or else item.is_space) then
				error_messages.extend ("ID does contain non-digits")
			elseif last_id = 0 then
				error_messages.extend ("ID must be a positive (> 0) value")
			end
		end

	parse_date is
			-- Parse a date from the input.
		local
			day_string,month_string,year_string: STRING
		do
			skip_white_space
			last_date := input.substring (position, position+9)
			if last_date.count < 10 then
				error_messages.extend ("date too short")
			elseif last_date.item (3) /= '.' or last_date.item (6) /= '.' then
				error_messages.extend ("dots not at the right positions in date")
			else
				day_string := last_date.substring (1, 2)
				month_string := last_date.substring (4, 5)
				year_string := last_date.substring (7, 10)
				if not (day_string.is_equal ("??") or day_string.is_natural) then
					error_messages.extend ("day needs to be a number or '??'")
				end
				if not (month_string.is_equal ("??") or month_string.is_natural) then
					error_messages.extend ("month needs to be a number or '??'")
				end
				if not (year_string.is_equal ("????") or year_string.is_natural) then
					error_messages.extend ("year needs to be a number or '????'")
				end
				if year_string.is_equal ("????") and not month_string.is_equal ("??") then
					error_messages.extend ("if the year is unknown, the month has to be unknown, too.")
				end
				if month_string.is_equal ("??") and not day_string.is_equal ("??") then
					error_messages.extend ("if the month is unknown, the day has to be unknown, too.")

				end
			end
			position := position + 10
			if not (end_of_input or else item.is_space) then
				error_messages.extend ("date too long")
			end
		end

	parse_name is
			-- Parse a name from the input.
		do
			skip_white_space
			if not end_of_input and item = '"' then
				from
					last_name := ""
					forth
				until
					item = '"' or end_of_input
				loop
					if item = '\' then
						forth
					end
					last_name.append_character (item)
					forth
				end
				if end_of_input then
					error_messages.extend ("end of input within string")
				else
					forth
				end
			else
				from
					last_name := ""
				until
					end_of_input or else item.is_space
				loop
					if item = '\' then
						forth
					end
					last_name.append_character (item)
					forth
				end
			end
			if last_name.is_equal ("") then
				error_messages.extend ("names must contain at least one character")
			end
		end

feature -- White space

	skip_white_space is
			-- Skip all white space characters.
		do
			from until
				end_of_input or else not item.is_space
			loop
				forth
			end
		end

invariant
	input_not_void: input /= Void
end
